# coding=utf-8
"""
Simulador Integral de Cadenas Productivas (SICP)

Copyleft (@) 2015 CENDITEL nodo Mérida - https://miv.cenditel.gob.ve/simulacion/
"""
## @package apps.comun.functions
#
# Funciones a implementar de uso común en el sistema
# @author Ing. Roldan Vargas (rvargas at cenditel.gob.ve)
# @author <a href='http://www.cenditel.gob.ve'>Centro Nacional de Desarrollo e Investigación en Tecnologías Libres
# (CENDITEL) nodo Mérida - Venezuela</a>
# @copyright <a href='http://www.gnu.org/licenses/gpl-2.0.html'>GNU Public License versión 2 (GPLv2)</a>
# @date 14-05-2015
# @version 3.0.0

## Muestra el autor del script cuando es invocado
__author__ = "Ing. Roldan Vargas (rvargas at cenditel.gob.ve)"
## Muestra documentación breve sobre el script
__doc__    = "Funciones a implementar de uso común en el sistema"

from django.core.mail import send_mail
from django.template.context import Context, RequestContext
from django.template.loader import get_template
from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from xml.etree.ElementTree import parse, SubElement

import smtplib
import logging
import os

## Variable que contiene el nombre del logger que registrará la vitácora de este módulo
logger = logging.getLogger("comun")

def enviar_correo(email, template, subject, vars=None):
    """!
    @brief Función que envía correos electrónicos
    @author Ing. Roldan Vargas (rvargas at cenditel.gob.ve)
    @copyright <a href='http://www.gnu.org/licenses/gpl-2.0.html'>GNU Public License versión 2 (GPLv2)</a>
    @date 14-05-2015
    @param email    <b>{string}</b> Dirección de correo electrónico del destinatario.
    @param template <b>{string}</b> Nombre de la plantilla de correo electrónico a utilizar.
    @param subject  <b>{string}</b> Texto del asunto que contendrá el correo electrónico.
    @param vars     <b>{object}</b> Diccionario de variables que serán pasadas a la plantilla de correo. El valor por defecto es Ninguno.
    @return Devuelve verdadero si el correo fue enviado, en caso contrario, devuelve falso
    """
    if not vars:
        vars = {}

    try:
        ## Obtiene la plantilla de correo a implementar
        t = get_template(template)
        c = Context(vars)
        send_mail(subject, t.render(c), settings.EMAIL_FROM, [email], fail_silently=False)
        logger.info(u"Correo enviado a %s usando la plantilla %s" % (email, template))
        return True
    except smtplib.SMTPException, e:
        logger.error(u"Ocurrió un error al enviar el correo a [%(correo)s]. Detalles del error: %(error)s"
                     % {'correo': email, 'error':e.message}, exc_info=True)
        return False


def render_cadena(usuario, findgraph="graph", findnodes="nodes", findarcs="arcs", findallnode="node", findallarc="arc"):
    """!
    @brief Función que obtiene los datos de una cadena de producción

    @author Ing. Roldan Vargas (rvargas at cenditel.gob.ve)
    @copyright <a href='http://www.gnu.org/licenses/gpl-2.0.html'>GNU Public License versión 2 (GPLv2)</a>
    @date 18-06-2015
    @param usuario      <b>{string}</b> Nombre del usuario autenticado en el sistema
    @param findgraph    <b>{string}</b> Nombre del gráfico a buscar dentro del xml de la cadena generada. El valor por defecto es 'graph'.
    @param findnodes    <b>{string}</b> Nombre de los nodos a buscar dentro de un gráfico. El valor por defecto es 'nodes'.
    @param findarcs     <b>{string}</b> Nombre de los arcos a buscar dentro de un gráfico. El valor por defecto es 'arcs'.
    @param findallnode  <b>{string}</b> Nombre del nodo a buscar dentro de los nodos de la cadena. El valor por defecto es 'node'.
    @param findallarc   <b>{string}</b> Nombre del arco a buscar dentro de los arcos de la cadena. El valor por defecto es 'arc'.
    @return Retorna un diccionario con los datos de la cadena de producción
    """
    xml_file = os.path.join(settings.BASE_DIR, "tmp/%s.xml" % usuario)
    tree = parse(xml_file)
    root = tree.getroot()
    graph = root.find(findgraph)
    nodes = graph.find(findnodes)
    arcs = graph.find(findarcs)
    node = nodes.findall(findallnode)
    arc = arcs.findall(findallarc)

    ## Inicialización de variables para determinar los niveles de la cadena
    narriba, nabajo = [], []

    ## Instrucción que permite obtener los niveles representados en la cadena aguas arriba y aguas abajo
    for a in arc:
        # Por cada arco de la cadena se inicializan las variables de niveles
        narriba = []
        nabajo = []
        for n in node:
            ## Condición que evalúa si contiene niveles aguas arriba
            if a.attrib['tgt'] == '0' and n.attrib['position'] == a.attrib['src'] and n.attrib['level'] not in narriba:
                ## Niveles Aguas Arriba
                narriba.append(n.attrib['level'])

            ## Condición que evalúa si contiene niveles aguas abajo
            elif a.attrib['src'] == '0' and n.attrib['position'] == a.attrib['tgt'] and n.attrib['level'] not in nabajo:
                ## Niveles Aguas Abajo
                nabajo.append(n.attrib['level'])

    attrs = root.find('attributes')

    anho = attrs.find('year').items()[0][1] ## Variable que contiene el año de consulta de la cadena productiva

    return {
        'node': node, 'arc': arc, 'narriba': narriba, 'nabajo': nabajo, 'anho': anho, 'graph': graph, 'tree': tree
    }


def render_simulacion(usuario, datos):
    """!
    Función que genera el archivo xml correspondiente a los datos a ser simulados

    @author Ing. Roldan Vargas (rvargas at cenditel.gob.ve)
    @copyright <a href='http://www.gnu.org/licenses/gpl-2.0.html'>GNU Public License versión 2 (GPLv2)</a>
    @date 18-06-2015
    @param usuario  <b>{string}</b> Nombre del usuario autenticado en el sistema
    @param datos    <b>{object}</b> Diccionario de datos con información del formulario para la construcción del escenario a simular
    """

    cadena = render_cadena(usuario)
    xml_file_out = os.path.join(settings.BASE_DIR, "tmp/%s%s.xml") % (str(usuario), "forSim")

    ev = SubElement(cadena['graph'], 'exogenous_variables')
    ev.set('average_salary', datos['salario'])
    ev.set('sigma', datos['desviacion'])
    ev.set('num_it', '3')

    ss0 = SubElement(ev, 'simulated_state')
    ss0.set('number','0')
    ss0.set('rate_of_change_of_salary','0')
    ss0.set('rate_of_gain','0')
    ss0.set('rate_of_price_change','0')
    ss0.set('rate_of_change_in_final_demand','0')
    ss0.set('nominal_exchange_rate','0')
    ss0.set('UT', datos['unidad_tributaria'])

    ss1 = SubElement(ev, 'simulated_state')
    ss1.set('number','1')
    ss1.set('rate_of_change_of_salary', datos['variacion_salario_0'])
    ss1.set('rate_of_gain', datos['tasa_ganancia_0'])
    ss1.set('rate_of_price_change', datos['variacion_precio_0'])
    ss1.set('rate_of_change_in_final_demand', datos['variacion_demanda_0'])
    ss1.set('nominal_exchange_rate', datos['cambio_nominal_0'])
    ss1.set('UT', datos['unidad_tributaria_0'])

    ss2 = SubElement(ev, 'simulated_state')
    ss2.set('number','2')
    ss2.set('rate_of_change_of_salary', datos['variacion_salario_1'])
    ss2.set('rate_of_gain', datos['tasa_ganancia_1'])
    ss2.set('rate_of_price_change', datos['variacion_precio_1'])
    ss2.set('rate_of_change_in_final_demand', datos['variacion_demanda_1'])
    ss2.set('nominal_exchange_rate', datos['cambio_nominal_1'])
    ss2.set('UT', datos['unidad_tributaria_1'])

    ss3 = SubElement(ev, 'simulated_state')
    ss3.set('number','3')
    ss3.set('rate_of_change_of_salary', datos['variacion_salario_2'])
    ss3.set('rate_of_gain', datos['tasa_ganancia_2'])
    ss3.set('rate_of_price_change', datos['variacion_precio_2'])
    ss3.set('rate_of_change_in_final_demand', datos['variacion_demanda_2'])
    ss3.set('nominal_exchange_rate', datos['cambio_nominal_2'])
    ss3.set('UT', datos['unidad_tributaria_2'])

    logger.info(
        u"Se ha generado un nuevo archivo para el proceso de simulación solicitado por el usuario %s" % str(usuario)
    )

    cadena['tree'].write(xml_file_out)